@@ -1,7 +1,7 @@
 \unset ECHO
 \i test/setup.sql
 
-SELECT plan(399);
+SELECT plan(459);
 --SELECT * FROM no_plan();
 
 -- This will be rolled back. :-)
@@ -69,6 +69,7 @@ CREATE TYPE someschema."myType" AS (
     foo INT
 );
 
+
 RESET client_min_messages;
 
 /****************************************************************************/
@@ -1224,7 +1225,8 @@ SELECT * FROM check_test(
 
 CREATE OR REPLACE FUNCTION ___mycasts(ex text) RETURNS TEXT[] AS $$
     SELECT ARRAY(
-        SELECT display_type(castsource, NULL) || ' AS ' || display_type(casttarget, NULL)
+        SELECT pg_catalog.format_type(castsource, NULL)
+               || ' AS ' || pg_catalog.format_type(casttarget, NULL)
           FROM pg_catalog.pg_cast
          WHERE castsource::regtype::text <> $1
     );
@@ -1459,6 +1461,469 @@ SELECT * FROM check_test(
 );
 
 /****************************************************************************/
+-- Test materialized_views_are().
+CREATE FUNCTION test_materialized_views_are() RETURNS SETOF TEXT AS $$
+DECLARE
+    tap record;
+BEGIN
+    IF pg_version_num() >= 90300 THEN
+        EXECUTE $E$
+            CREATE MATERIALIZED VIEW public.moo AS SELECT * FROM foo;
+            CREATE MATERIALIZED VIEW public.mou AS SELECT * FROM fou;
+        $E$;
+        
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( 'public', ARRAY['mou', 'moo'], 'whatever' ),
+            true,
+            'materialized_views_are(schema, materialized_views, desc)',
+            'whatever',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( 'public', ARRAY['mou', 'moo'] ),
+            true,
+            'materialized_views_are(schema, materialized_views)',
+            'Schema public should have the correct materialized views',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( ARRAY['mou', 'moo'] ),
+            true,
+            'materialized_views_are(views)',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct materialized views',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( ARRAY['mou', 'moo'], 'whatever' ),
+            true,
+            'materialized_views_are(views, desc)',
+            'whatever',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( 'public', ARRAY['mou', 'moo', 'bar'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) missing',
+            'Schema public should have the correct materialized views',
+            '    Missing Materialized views:
+        bar'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( ARRAY['mou', 'moo', 'bar'] ),
+            false,
+            'materialized_views_are(materialized_views) missing',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct materialized views',
+            '    Missing Materialized views:
+        bar'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( 'public', ARRAY['mou'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) extra',
+            'Schema public should have the correct materialized views',
+            '    Extra Materialized views:
+        moo'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( ARRAY['mou'] ),
+            false,
+            'materialized_views_are(materialized_views) extra',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct materialized views',
+            '    Extra Materialized views:
+        moo'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( 'public', ARRAY['bar', 'baz'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) extra and missing',
+            'Schema public should have the correct materialized views',
+            '    Extra Materialized views:
+        mo[ou]
+        mo[ou]
+    Missing Materialized views:
+        ba[rz]
+        ba[rz]',
+            true
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            materialized_views_are( ARRAY['bar', 'baz'] ),
+            false,
+            'materialized_views_are(materialized_views) extra and missing',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct materialized views',
+            '    Extra Materialized views:' || '
+        mo[ou]
+        mo[ou]
+    Missing Materialized views:' || '
+        ba[rz]
+        ba[rz]',
+            true
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    ELSE
+        -- Fake it with views_are
+        FOR tap IN SELECT * FROM check_test(
+            views_are( 'public', ARRAY['vou', 'voo'], 'whatever' ),
+            true,
+            'materialized_views_are(schema, materialized_views, desc)',
+            'whatever',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( 'public', ARRAY['vou', 'voo'] ),
+            true,
+            'materialized_views_are(schema, materialized_views)',
+            'Schema public should have the correct views',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( ARRAY['vou', 'voo'] ),
+            true,
+            'materialized_views_are(views)',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct views',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( ARRAY['vou', 'voo'], 'whatever' ),
+            true,
+            'materialized_views_are(views, desc)',
+            'whatever',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( 'public', ARRAY['vou', 'voo', 'bar'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) missing',
+            'Schema public should have the correct views',
+            '    Missing views:
+        bar'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( ARRAY['vou', 'voo', 'bar'] ),
+            false,
+            'materialized_views_are(materialized_views) missing',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct views',
+            '    Missing views:
+        bar'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( 'public', ARRAY['vou'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) extra',
+            'Schema public should have the correct views',
+            '    Extra views:
+        voo'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( ARRAY['vou'] ),
+            false,
+            'materialized_views_are(materialized_views) extra',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct views',
+            '    Extra views:
+        voo'
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( 'public', ARRAY['bar', 'baz'] ),
+            false,
+            'materialized_views_are(schema, materialized_views) extra and missing',
+            'Schema public should have the correct views',
+            '    Extra views:
+        vo[ou]
+        vo[ou]
+    Missing views:
+        ba[rz]
+        ba[rz]',
+            true
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            views_are( ARRAY['bar', 'baz'] ),
+            false,
+            'materialized_views_are(materialized_views) extra and missing',
+            'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct views',
+            '    Extra views:' || '
+        vo[ou]
+        vo[ou]
+    Missing views:' || '
+        ba[rz]
+        ba[rz]',
+            true
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    end IF;
+    
+    RETURN;
+END; 
+$$LANGUAGE PLPGSQL;
+
+SELECT * FROM test_materialized_views_are();
+
+/****************************************************************************/
+-- tests for foreign_tables_are functions
+
+CREATE FUNCTION test_foreign_tables_are() RETURNS SETOF TEXT AS $$
+DECLARE
+    tap record;
+    i int;
+BEGIN
+    IF pg_version_num() >= 90100 THEN
+        -- setup tables for tests
+        EXECUTE $setup$
+        CREATE FOREIGN DATA WRAPPER null_fdw;
+        CREATE SERVER server_null_fdw FOREIGN DATA WRAPPER null_fdw;
+        CREATE FOREIGN TABLE public.ft_foo(
+            id      INT ,
+            "name"  TEXT ,
+            num     NUMERIC(10, 2),
+          "myInt" int
+        )
+        SERVER server_null_fdw ;
+        CREATE FOREIGN TABLE public.ft_bar(
+            id    INT 
+        )
+        SERVER server_null_fdw ;
+        $setup$;
+
+-- foreign_tables_are( schema, tables, description )
+-- CREATE OR REPLACE FUNCTION foreign_tables_are ( NAME, NAME[], TEXT )
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( 'public', ARRAY['ft_foo', 'ft_bar'], 'Correct foreign tables in named schema with custom message' ),
+    true,
+    'foreign_tables_are(schema, tables, desc)',
+    'Correct foreign tables in named schema with custom message',
+    ''
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+-- foreign_tables_are( tables, description )
+--CREATE OR REPLACE FUNCTION foreign_tables_are ( NAME[], TEXT )
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( ARRAY['ft_foo', 'ft_bar'], 'Correct foreign tables in search_path with custom message' ),
+    true,
+    'foreign_tables_are(tables, desc)',
+    'Correct foreign tables in search_path with custom message',
+    ''
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+
+-- foreign_tables_are( schema, tables )
+-- CREATE OR REPLACE FUNCTION foreign_tables_are ( NAME, NAME[] )
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( 'public', ARRAY['ft_foo', 'ft_bar'] ),
+    true,
+    'foreign_tables_are(schema, tables)',
+    'Schema public should have the correct foreign tables',
+    ''
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+-- foreign_tables_are( tables )
+-- +CREATE OR REPLACE FUNCTION foreign_tables_are ( NAME[] )
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( ARRAY['ft_foo', 'ft_bar'] ),
+    true,
+    'foreign_tables_are(tables)',
+    'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct foreign tables',
+    ''
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( 'public', ARRAY['ft_foo', 'ft_bar', 'bar'] ),
+    false,
+    'foreign_tables_are(schema, tables) missing',
+    'Schema public should have the correct foreign tables',
+    '    Missing foreign tables:
+        bar'
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( ARRAY['ft_foo', 'ft_bar', 'bar'] ),
+    false,
+    'foreign_tables_are(tables) missing',
+    'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct foreign tables',
+    '    Missing foreign tables:
+        bar'
+      ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( 'public', ARRAY['ft_foo'] ),
+    false,
+    'foreign_tables_are(schema, tables) extra',
+    'Schema public should have the correct foreign tables',
+    '    Extra foreign tables:
+        ft_bar'
+      ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( ARRAY['ft_foo'] ),
+    false,
+    'foreign_tables_are(tables) extra',
+    'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct foreign tables',
+    '    Extra foreign tables:
+        ft_bar'
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( 'public', ARRAY['bar', 'baz'] ),
+    false,
+    'foreign_tables_are(schema, tables) extra and missing',
+    'Schema public should have the correct foreign tables',
+    '    Extra foreign tables:
+        ft_foo
+        ft_bar
+    Missing foreign tables:
+        ba[rz]
+        ba[rz]',
+    true
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+   FOR tap IN SELECT * FROM check_test(
+    foreign_tables_are( ARRAY['bar', 'baz'] ),
+    false,
+    'foreign_tables_are(tables) extra and missing',
+    'Search path ' || pg_catalog.current_setting('search_path') || ' should have the correct foreign tables',
+    '    Extra foreign tables:' || '
+        ft_[fobar]+
+        ft_[fobar]+
+    Missing foreign tables:' || '
+        ba[rz]
+        ba[rz]',
+    true
+    ) AS r LOOP
+            RETURN NEXT tap.r;
+        END LOOP;
+
+    else
+       -- 10 fake pass/fail tests to make test work in older postgres. 
+       FOR tap IN SELECT * FROM check_test(pass('pass'), true,
+                'foreign_tables_are(schema, tables, desc)'
+            , 'pass', '') AS r LOOP
+           return next tap.r;
+       end loop;
+       FOR tap IN SELECT * FROM check_test(pass('pass'), true,
+                'foreign_tables_are(tables, desc)'
+            , 'pass', '') AS r LOOP
+           return next tap.r;
+       end loop;
+       FOR tap IN SELECT * FROM check_test(pass('pass'), true, 
+        'foreign_tables_are(schema, tables)'
+        ,'pass' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(pass('pass'), true, 
+        'foreign_tables_are(tables)'
+        ,'pass' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(schema, tables) missing'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(tables) missing'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(schema, tables) extra'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(tables) extra'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(schema, tables) extra and missing'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+       FOR tap IN SELECT * FROM check_test(fail('fail'), false,
+        'foreign_tables_are(tables) extra and missing'
+        ,'fail' ,'') AS r LOOP
+           RETURN NEXT tap.r;
+       END LOOP;
+    end if;
+    RETURN;
+END;
+$$ LANGUAGE PLPGSQL;
+
+SELECT * FROM test_foreign_tables_are();
+
+/****************************************************************************/
+
 -- Finish the tests and clean up.
 SELECT * FROM finish();
 ROLLBACK;
