@@ -1,7 +1,7 @@
 \unset ECHO
 \i test/setup.sql
 
-SELECT plan(466);
+SELECT plan(520);
 --SELECT * FROM no_plan();
 
 CREATE SCHEMA someschema;
@@ -983,7 +983,7 @@ SELECT * FROM check_test(
 );
 
 /****************************************************************************/
--- Test is_strict().
+-- Test is_strict() and isnt_strict().
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', '{}'::name[], 'whatever' ),
     true,
@@ -992,6 +992,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', '{}'::name[], 'whatever' ),
+    false,
+    'isnt_strict(schema, func, 0 args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', '{}'::name[] ),
     true,
@@ -1000,6 +1008,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', '{}'::name[] ),
+    false,
+    'isnt_strict(schema, func, 0 args)',
+    'Function public.yay() should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'oww', ARRAY['integer', 'text'], 'whatever' ),
     false,
@@ -1008,6 +1024,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'oww', ARRAY['integer', 'text'], 'whatever' ),
+    true,
+    'is_strict(schema, func, args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'oww', ARRAY['integer', 'text'] ),
     false,
@@ -1016,6 +1040,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'oww', ARRAY['integer', 'text'] ),
+    true,
+    'is_strict(schema, func, args)',
+    'Function public.oww(integer, text) should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', 'whatever' ),
     true,
@@ -1024,6 +1056,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', 'whatever' ),
+    false,
+    'isnt_strict(schema, func, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay'::name ),
     true,
@@ -1032,6 +1072,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay'::name ),
+    false,
+    'isnt_strict(schema, func)',
+    'Function public.yay() should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', '{}'::name[], 'whatever' ),
     true,
@@ -1040,6 +1088,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', '{}'::name[], 'whatever' ),
+    false,
+    'isnt_strict(schema, func, 0 args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', '{}'::name[] ),
     true,
@@ -1048,6 +1104,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', '{}'::name[] ),
+    false,
+    'isnt_strict(schema, func, 0 args)',
+    'Function public.yay() should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'oww', ARRAY['integer', 'text'], 'whatever' ),
     false,
@@ -1056,6 +1120,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'oww', ARRAY['integer', 'text'], 'whatever' ),
+    true,
+    'isnt_strict(schema, func, args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'oww', ARRAY['integer', 'text'] ),
     false,
@@ -1064,6 +1136,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'oww', ARRAY['integer', 'text'] ),
+    true,
+    'isnt_strict(schema, func, args)',
+    'Function public.oww(integer, text) should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay', 'whatever' ),
     true,
@@ -1072,6 +1152,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay', 'whatever' ),
+    false,
+    'isnt_strict(schema, func, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'public', 'yay'::name ),
     true,
@@ -1080,6 +1168,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'public', 'yay'::name ),
+    false,
+    'isnt_strict(schema, func)',
+    'Function public.yay() should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'yay', '{}'::name[], 'whatever' ),
     true,
@@ -1088,6 +1184,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'yay', '{}'::name[], 'whatever' ),
+    false,
+    'isnt_strict(func, 0 args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'yay', '{}'::name[] ),
     true,
@@ -1096,6 +1200,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'yay', '{}'::name[] ),
+    false,
+    'isnt_strict(func, 0 args)',
+    'Function yay() should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'oww', ARRAY['integer', 'text'], 'whatever' ),
     false,
@@ -1104,6 +1216,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'oww', ARRAY['integer', 'text'], 'whatever' ),
+    true,
+    'isnt_strict(func, args, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'oww', ARRAY['integer', 'text'] ),
     false,
@@ -1112,6 +1232,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'oww', ARRAY['integer', 'text'] ),
+    true,
+    'isnt_strict(func, args)',
+    'Function oww(integer, text) should not be strict',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'yay', 'whatever' ),
     true,
@@ -1120,6 +1248,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'yay', 'whatever' ),
+    false,
+    'isnt_strict(func, desc)',
+    'whatever',
+    ''
+);
+
 SELECT * FROM check_test(
     is_strict( 'yay'::name ),
     true,
@@ -1128,6 +1264,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    isnt_strict( 'yay'::name ),
+    false,
+    'isnt_strict(func)',
+    'Function yay() should not be strict',
+    ''
+);
+
 /****************************************************************************/
 -- Test volatility_is().
 SELECT * FROM check_test(
