@@ -1,7 +1,7 @@
 \unset ECHO
 \i test/setup.sql
 
-SELECT plan(678);
+SELECT plan(830);
 --SELECT * FROM no_plan();
 
 -- This will be rolled back. :-)
@@ -32,6 +32,7 @@ CREATE DOMAIN public."myDomain" AS TEXT CHECK(TRUE);
 CREATE SEQUENCE public.someseq;
 
 CREATE SCHEMA someschema;
+
 RESET client_min_messages;
 
 /****************************************************************************/
@@ -175,9 +176,17 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    has_table( '__SDFSDFD__', 'lol' ),
+    has_table( '__SDFSDFD__', 'lol'::name ),
     false,
     'has_table(non-existent schema, tab)',
+    'Table "__SDFSDFD__".lol should exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_table( '__SDFSDFD__', 'lol' ),
+    false,
+    'has_table(non-existent table, desc)',
     'lol',
     ''
 );
@@ -234,9 +243,17 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    hasnt_table( '__SDFSDFD__', 'lol' ),
+    hasnt_table( '__SDFSDFD__', 'lol'::name ),
     true,
     'hasnt_table(non-existent schema, tab)',
+    'Table "__SDFSDFD__".lol should not exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_table( '__SDFSDFD__', 'lol' ),
+    true,
+    'hasnt_table(non-existent table, desc)',
     'lol',
     ''
 );
@@ -394,6 +411,14 @@ SELECT * FROM check_test(
     ''
 );
 
+SELECT * FROM check_test(
+    has_sequence( 'public', 'someseq'::name ),
+    true,
+    'has_sequence(sch, sequence)',
+    'Sequence public.someseq should exist'
+    ''
+);
+
 /****************************************************************************/
 -- Test hasnt_sequence().
 
@@ -437,6 +462,108 @@ SELECT * FROM check_test(
     ''
 );
 
+/****************************************************************************/
+-- Test has_composite().
+
+SELECT * FROM check_test(
+    has_composite( '__SDFSDFD__' ),
+    false,
+    'has_composite(non-existent composite type)',
+    'Composite type "__SDFSDFD__" should exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_composite( '__SDFSDFD__', 'lol' ),
+    false,
+    'has_composite(non-existent schema, tab)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_composite( 'foo', '__SDFSDFD__', 'desc' ),
+    false,
+    'has_composite(sch, non-existent composite type, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_composite( 'sometype', 'lol' ),
+    true,
+    'has_composite(tab, desc)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_composite( 'public', 'sometype', 'desc' ),
+    true,
+    'has_composite(sch, tab, desc)',
+    'desc',
+    ''
+);
+
+-- It should ignore views and tables.
+SELECT * FROM check_test(
+    has_composite( 'pg_catalog', 'pg_composites', 'desc' ),
+    false,
+    'has_composite(sch, view, desc)',
+    'desc',
+    ''
+);
+SELECT * FROM check_test(
+    has_composite( 'sometab', 'desc' ),
+    false,
+    'has_composite(type, desc)',
+    'desc',
+    ''
+);
+
+/****************************************************************************/
+-- Test hasnt_composite().
+
+SELECT * FROM check_test(
+    hasnt_composite( '__SDFSDFD__' ),
+    true,
+    'hasnt_composite(non-existent composite type)',
+    'Composite type "__SDFSDFD__" should not exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_composite( '__SDFSDFD__', 'lol' ),
+    true,
+    'hasnt_composite(non-existent schema, tab)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_composite( 'foo', '__SDFSDFD__', 'desc' ),
+    true,
+    'hasnt_composite(sch, non-existent tab, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_composite( 'sometype', 'lol' ),
+    false,
+    'hasnt_composite(tab, desc)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_composite( 'public', 'sometype', 'desc' ),
+    false,
+    'hasnt_composite(sch, tab, desc)',
+    'desc',
+    ''
+);
+
 /****************************************************************************/
 -- Test has_type().
 SELECT * FROM check_test(
@@ -1647,7 +1774,7 @@ SELECT * FROM check_test(
 /****************************************************************************/
 -- Test domain_type_is() and domain_type_isnt().
 SELECT * FROM check_test(
-    domain_type_is( 'public', 'us_postal_code', 'public', 'text', 'whatever'),
+    domain_type_is( 'public', 'us_postal_code', 'pg_catalog', 'text', 'whatever'),
     true,
     'domain_type_is(schema, domain, schema, type, desc)',
     'whatever',
@@ -1655,24 +1782,24 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    domain_type_is( 'public', 'us_postal_code', 'public'::name, 'text'),
+    domain_type_is( 'public', 'us_postal_code', 'pg_catalog'::name, 'text'),
     true,
     'domain_type_is(schema, domain, schema, type)',
-    'Domain public.us_postal_code should extend type public.text',
+    'Domain public.us_postal_code should extend type pg_catalog.text',
     ''
 );
 
 SELECT * FROM check_test(
-    domain_type_is( 'public', 'us_postal_code', 'public', 'integer', 'whatever'),
+    domain_type_is( 'public', 'us_postal_code', 'pg_catalog', 'integer', 'whatever'),
     false,
     'domain_type_is(schema, domain, schema, type, desc) fail',
     'whatever',
-    '        have: public.text
-        want: public.integer'
+    '        have: pg_catalog.text
+        want: pg_catalog.integer'
 );
 
 SELECT * FROM check_test(
-    domain_type_is( 'public', 'zip_code', 'public', 'integer', 'whatever'),
+    domain_type_is( 'public', 'zip_code', 'pg_catalog', 'integer', 'whatever'),
     false,
     'domain_type_is(schema, nondomain, schema, type, desc)',
     'whatever',
@@ -1680,7 +1807,7 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    domain_type_is( 'public', 'integer', 'public', 'integer', 'whatever'),
+    domain_type_is( 'public', 'integer', 'pg_catalog', 'integer', 'whatever'),
     false,
     'domain_type_is(schema, type, schema, type, desc) fail',
     'whatever',
@@ -1778,24 +1905,24 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    domain_type_isnt( 'public', 'us_postal_code', 'public'::name, 'integer'),
+    domain_type_isnt( 'public', 'us_postal_code', 'pg_catalog'::name, 'integer'),
     true,
     'domain_type_isnt(schema, domain, schema, type)',
-    'Domain public.us_postal_code should not extend type public.integer',
+    'Domain public.us_postal_code should not extend type pg_catalog.integer',
     ''
 );
 
 SELECT * FROM check_test(
-    domain_type_isnt( 'public', 'us_postal_code', 'public', 'text', 'whatever'),
+    domain_type_isnt( 'public', 'us_postal_code', 'pg_catalog', 'text', 'whatever'),
     false,
     'domain_type_isnt(schema, domain, schema, type, desc) fail',
     'whatever',
-    '        have: public.text
+    '        have: pg_catalog.text
         want: anything else'
 );
 
 SELECT * FROM check_test(
-    domain_type_isnt( 'public', 'zip_code', 'public', 'text', 'whatever'),
+    domain_type_isnt( 'public', 'zip_code', 'pg_catalog', 'text', 'whatever'),
     false,
     'domain_type_isnt(schema, nondomain, schema, type, desc)',
     'whatever',
@@ -1803,7 +1930,7 @@ SELECT * FROM check_test(
 );
 
 SELECT * FROM check_test(
-    domain_type_isnt( 'public', 'integer', 'public', 'text', 'whatever'),
+    domain_type_isnt( 'public', 'integer', 'pg_catalog', 'text', 'whatever'),
     false,
     'domain_type_isnt(schema, type, schema, type, desc)',
     'whatever',
@@ -1892,7 +2019,647 @@ SELECT * FROM check_test(
     '   Domain integer does not exist'
 );
 
+/****************************************************************************/
+-- Test has_foreign_table() and hasnt_foreign_table().
+CREATE FUNCTION test_fdw() RETURNS SETOF TEXT AS $$
+DECLARE
+    tap record;
+BEGIN   
+    IF pg_version_num() >= 90100 THEN
+        EXECUTE $E$
+            CREATE FOREIGN DATA WRAPPER dummy;
+            CREATE SERVER foo FOREIGN DATA WRAPPER dummy;
+            CREATE FOREIGN TABLE public.my_fdw (id int) SERVER foo;
+        $E$;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( '__SDFSDFD__' ),
+            false,
+            'has_foreign_table(non-existent table)',
+            'Foreign table "__SDFSDFD__" should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( '__SDFSDFD__', 'lol'::name ),
+            false,
+            'has_foreign_table(non-existent schema, tab)',
+            'Foreign table "__SDFSDFD__".lol should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( '__SDFSDFD__', 'lol' ),
+            false,
+            'has_foreign_table(non-existent table, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( 'foo', '__SDFSDFD__', 'desc' ),
+            false,
+            'has_foreign_table(sch, non-existent table, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( 'my_fdw', 'lol' ),
+            true,
+            'has_foreign_table(tab, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( 'public', 'my_fdw', 'desc' ),
+            true,
+            'has_foreign_table(sch, tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        -- It should ignore views and types.
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( 'pg_catalog', 'pg_foreign_tables', 'desc' ),
+            false,
+            'has_foreign_table(sch, view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_foreign_table( 'sometype', 'desc' ),
+            false,
+            'has_foreign_table(type, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( '__SDFSDFD__' ),
+            true,
+            'hasnt_foreign_table(non-existent table)',
+            'Foreign table "__SDFSDFD__" should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( '__SDFSDFD__', 'lol'::name ),
+            true,
+            'hasnt_foreign_table(non-existent schema, tab)',
+            'Foreign table "__SDFSDFD__".lol should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( '__SDFSDFD__', 'lol' ),
+            true,
+            'hasnt_foreign_table(non-existent table, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( 'foo', '__SDFSDFD__', 'desc' ),
+            true,
+            'hasnt_foreign_table(sch, non-existent tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( 'my_fdw', 'lol' ),
+            false,
+            'hasnt_foreign_table(tab, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_foreign_table( 'public', 'my_fdw', 'desc' ),
+            false,
+            'hasnt_foreign_table(sch, tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    ELSE
+        -- Fake it with has_table().
+        FOR tap IN SELECT * FROM check_test(
+            has_table( '__SDFSDFD__' ),
+            false,
+            'has_foreign_table(non-existent table)',
+            'Table "__SDFSDFD__" should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( '__SDFSDFD__', 'lol'::name ),
+            false,
+            'has_foreign_table(non-existent schema, tab)',
+            'Table "__SDFSDFD__".lol should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( '__SDFSDFD__', 'lol' ),
+            false,
+            'has_foreign_table(non-existent table, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( 'foo', '__SDFSDFD__', 'desc' ),
+            false,
+            'has_foreign_table(sch, non-existent table, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( 'pg_type', 'lol' ),
+            true,
+            'has_foreign_table(tab, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( 'pg_catalog', 'pg_type', 'desc' ),
+            true,
+            'has_foreign_table(sch, tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        -- It should ignore views and types.
+        FOR tap IN SELECT * FROM check_test(
+            has_table( 'pg_catalog', 'pg_foreign_tables', 'desc' ),
+            false,
+            'has_foreign_table(sch, view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_table( 'sometype', 'desc' ),
+            false,
+            'has_foreign_table(type, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( '__SDFSDFD__' ),
+            true,
+            'hasnt_foreign_table(non-existent table)',
+            'Table "__SDFSDFD__" should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( '__SDFSDFD__', 'lol'::name ),
+            true,
+            'hasnt_foreign_table(non-existent schema, tab)',
+            'Table "__SDFSDFD__".lol should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( '__SDFSDFD__', 'lol' ),
+            true,
+            'hasnt_foreign_table(non-existent table, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( 'foo', '__SDFSDFD__', 'desc' ),
+            true,
+            'hasnt_foreign_table(sch, non-existent tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( 'pg_type', 'lol' ),
+            false,
+            'hasnt_foreign_table(tab, desc)',
+            'lol',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_table( 'pg_catalog', 'pg_type', 'desc' ),
+            false,
+            'hasnt_foreign_table(sch, tab, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    END IF;
+    RETURN;
+END;
+$$ LANGUAGE PLPGSQL;
+
+/****************************************************************************/
+-- Test has_relation().
+
+SELECT * FROM check_test(
+    has_relation( '__SDFSDFD__' ),
+    false,
+    'has_relation(non-existent relation)',
+    'Relation "__SDFSDFD__" should exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_relation( '__SDFSDFD__', 'lol' ),
+    false,
+    'has_relation(non-existent schema, tab)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_relation( 'foo', '__SDFSDFD__', 'desc' ),
+    false,
+    'has_relation(sch, non-existent relation, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_relation( 'pg_type', 'lol' ),
+    true,
+    'has_relation(tab, desc)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_relation( 'pg_catalog', 'pg_type', 'desc' ),
+    true,
+    'has_relation(sch, tab, desc)',
+    'desc',
+    ''
+);
+
+-- It should not ignore views and types.
+SELECT * FROM check_test(
+    has_relation( 'pg_catalog', 'pg_type', 'desc' ),
+    true,
+    'has_relation(sch, view, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM check_test(
+    has_relation( 'sometype', 'desc' ),
+    true,
+    'has_relation(type, desc)',
+    'desc',
+    ''
+);
+
+/****************************************************************************/
+-- Test hasnt_relation().
+
+SELECT * FROM check_test(
+    hasnt_relation( '__SDFSDFD__' ),
+    true,
+    'hasnt_relation(non-existent relation)',
+    'Relation "__SDFSDFD__" should not exist',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_relation( '__SDFSDFD__', 'lol' ),
+    true,
+    'hasnt_relation(non-existent schema, tab)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_relation( 'foo', '__SDFSDFD__', 'desc' ),
+    true,
+    'hasnt_relation(sch, non-existent tab, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_relation( 'pg_type', 'lol' ),
+    false,
+    'hasnt_relation(tab, desc)',
+    'lol',
+    ''
+);
+
+SELECT * FROM check_test(
+    hasnt_relation( 'pg_catalog', 'pg_type', 'desc' ),
+    false,
+    'hasnt_relation(sch, tab, desc)',
+    'desc',
+    ''
+);
+
+SELECT * FROM test_fdw();
+
+/****************************************************************************/
+-- Test has_materialized_view().
+
+CREATE FUNCTION test_has_materialized_view() RETURNS SETOF TEXT AS $$
+DECLARE
+    tap record;
+BEGIN
+    IF pg_version_num() >= 93000 THEN
+        EXECUTE $E$
+            CREATE MATERIALIZED VIEW public.mview AS SELECT * FROM public.sometab;
+        $E$;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_materialized_view( '__SDFSDFD__' ),
+            false,
+            'has_materialized_view(non-existent materialized_view)',
+            'Materialized view "__SDFSDFD__" should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_materialized_view( '__SDFSDFD__', 'howdy' ),
+            false,
+            'has_materialized_view(non-existent materialized_view, desc)',
+            'howdy',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_materialized_view( 'foo', '__SDFSDFD__', 'desc' ),
+            false,
+            'has_materialized_view(sch, non-existtent materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_materialized_view( 'mview', 'yowza' ),
+            true,
+            'has_materialized_view(materialized_viewiew, desc)',
+            'yowza',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_materialized_view( 'public', 'mview', 'desc' ),
+            true,
+            'has_materialized_view(sch, materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+    ELSE
+        FOR tap IN SELECT * FROM check_test(
+            has_view( '__SDFSDFD__' ),
+            false,
+            'has_materialized_view(non-existent materialized_view)',
+            'View "__SDFSDFD__" should exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_view( '__SDFSDFD__', 'howdy' ),
+            false,
+            'has_materialized_view(non-existent materialized_view, desc)',
+            'howdy',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_view( 'foo', '__SDFSDFD__', 'desc' ),
+            false,
+            'has_materialized_view(sch, non-existtent materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_view( 'pg_tables', 'yowza' ),
+            true,
+            'has_materialized_view(materialized_view, desc)',
+            'yowza',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            has_view( 'information_schema', 'tables', 'desc' ),
+            true,
+            'has_materialized_view(sch, materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+        
+    end if;
+
+    return;
+end; $$language PLPGSQL;
+
+/****************************************************************************/
+-- Test hasnt_materialized_view().
+CREATE FUNCTION test_hasnt_materialized_views_are() RETURNS SETOF TEXT AS $$
+DECLARE
+    tap record;
+BEGIN
+    IF pg_version_num() >= 93000 THEN
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_materialized_view( '__SDFSDFD__' ),
+            true,
+            'hasnt_materialized_view(non-existent materialized_view)',
+            'Materialized view "__SDFSDFD__" should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_materialized_view( '__SDFSDFD__', 'howdy' ),
+            true,
+            'hasnt_materialized_view(non-existent materialized_view, desc)',
+            'howdy',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_materialized_view( 'foo', '__SDFSDFD__', 'desc' ),
+            true,
+            'hasnt_materialized_view(sch, non-existtent materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_materialized_view( 'mview', 'yowza' ),
+            false,
+            'hasnt_materialized_view(materialized_viewiew, desc)',
+            'yowza',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_materialized_view( 'public', 'mview', 'desc' ),
+            false,
+            'hasnt_materialized_view(sch, materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    else
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_view( '__SDFSDFD__' ),
+            true,
+            'hasnt_materialized_view(non-existent materialized_view)',
+            'View "__SDFSDFD__" should not exist',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_view( '__SDFSDFD__', 'howdy' ),
+            true,
+            'hasnt_materialized_view(non-existent materialized_view, desc)',
+            'howdy',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_view( 'foo', '__SDFSDFD__', 'desc' ),
+            true,
+            'hasnt_materialized_view(sch, non-existtent materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_view( 'pg_tables', 'yowza' ),
+            false,
+            'hasnt_materialized_view(materialized_view, desc)',
+            'yowza',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+
+        FOR tap IN SELECT * FROM check_test(
+            hasnt_view( 'information_schema', 'tables', 'desc' ),
+            false,
+            'hasnt_materialized_view(sch, materialized_view, desc)',
+            'desc',
+            ''
+        ) AS b LOOP
+            RETURN NEXT tap.b;
+        END LOOP;
+    end if;
+
+    return;
+end; $$language PLPGSQL;
+
+SELECT * FROM test_has_materialized_view();
+
+SELECT * FROm test_hasnt_materialized_views_are();
+
 /****************************************************************************/
 -- Finish the tests and clean up.
 SELECT * FROM finish();
 ROLLBACK;
+
